import json

import requests

from .constants.cli_constants import CliConstants


class ZephyrService:
    def __init__(self, zephyr_configurer):
        self.zephyr_config = zephyr_configurer.get_config()
        self.jira_url = self.zephyr_config.get(CliConstants.JIRA_URL.value)
        self.username = self.zephyr_config.get(CliConstants.USERNAME.value)
        self.password = self.zephyr_config.get(CliConstants.PASSWORD.value)
        self.project_key = self.zephyr_config.get(CliConstants.PROJECT_KEY.value)
        self.cycle_name = self.zephyr_config.get(CliConstants.TEST_CYCLE.value)
        self.folder_name = self.zephyr_config.get(CliConstants.FOLDER_NAME.value)
        self.version_name = self.zephyr_config.get(CliConstants.RELEASE_VERSION.value)
        self.single_update_uri = 'zapi/latest/execution/{}/execute'
        self.auth = (self.username, self.password)

    def get_zephyr_config(self):
        return self.zephyr_config

    def get_issue_by_key(self, issue_key):
        url = self.jira_url + "api/2/issue/" + issue_key

        print(f"Get {url}")
        resp = requests.get(url, auth=self.auth)

        if resp.status_code != 200:
            raise Exception(resp.content)

        return resp.json()['id']

    def create_new_execution(self, issue_id, zephyr_meta_info):
        url = self.jira_url + "zapi/latest/execution"

        headers = {'content-type': 'application/json'}
        post_data = json.dumps({
            "cycleId": zephyr_meta_info.get("cycleId"),
            "projectId": zephyr_meta_info.get("projectId"),
            "versionId": zephyr_meta_info.get("versionId"),
            "assigneeType": "assignee",
            "assignee": self.zephyr_config.get(CliConstants.USERNAME.value),
            "folderId": zephyr_meta_info.get("folderId"),
            "issueId": issue_id
        })

        print(f"Post {url} with data {post_data}")
        resp = requests.post(url, data=post_data, headers=headers, auth=self.auth)

        if resp.status_code != 200:
            raise Exception(resp.content)

        keys = resp.json().keys()

        return list(keys)[0]

    def update_execution(self, issue_key, status_number, comment=None):
        issue_id = self.get_issue_by_key(issue_key)

        url = self.jira_url + self.single_update_uri.format(str(issue_id))

        if comment is None:
            data = json.dumps({"status": status_number})
        else:
            data = json.dumps({
                "status": status_number,
                "comment": comment
            })
        headers = {'content-type': 'application/json'}

        print(f"Put {url} with data {data}")
        resp = requests.put(url, headers=headers, data=data, auth=self.auth)

        if resp.status_code != 200:
            raise Exception(resp.content)

    def get_project_id_by_key(self, project_key=None):
        if project_key is not None:
            p_key = project_key
        else:
            p_key = self.project_key

        url = self.jira_url + 'api/2/project/' + p_key
        headers = {
            'content-type': 'application/json',
            'accept': 'application/json'
        }
        print(f"Get {url}")
        project = requests.get(url, headers=headers, auth=self.auth)
        return project.json()['id']

    def get_version_for_project_id(self, version_name=None, project_id=None):
        if version_name is not None:
            v_name = version_name
        else:
            v_name = self.version_name

        if project_id is not None:
            p_id = project_id
        else:
            p_id = self.get_project_id_by_key(self.project_key)

        url = self.jira_url + 'zapi/latest/util/versionBoard-list?projectId=' + p_id

        headers = {'accept': 'application/json'}

        print(f"Get {url}")
        project = requests.get(url, headers=headers, auth=self.auth)
        for version in project.json()['unreleasedVersions']:
            if version['label'] == v_name:
                return version['value']

    def create_test_cycle(self, cycle_name, version_id=None, project_id=None, description=None):
        url = self.jira_url + 'zapi/latest/cycle'

        if version_id is not None:
            v_id = version_id
        else:
            v_id = self.self.get_version_for_project_id()

        if project_id is not None:
            p_id = project_id
        else:
            p_id = self.get_project_id_by_key(self.project_key)

        headers = {'content-type': 'application/json'}
        post_data = json.dumps({
            "clonedCycleId": "",
            "name": cycle_name,
            "build": "",
            "environment": "",
            "description": description,
            "projectId": p_id,
            "versionId": v_id
        })

        print(f"Post {url} with data {post_data}")
        resp = requests.post(url, data=post_data, headers=headers, auth=self.auth)

        return resp.json()['id']

    def get_cycle_id(self, cycle_name=None, project_id=None, version_id=None):
        if project_id is not None:
            p_id = project_id
        else:
            p_id = self.get_project_id_by_key(self.project_key)

        if version_id is not None:
            v_id = version_id
        else:
            v_id = self.get_version_for_project_id()

        url = self.jira_url + 'zapi/latest/cycle?projectId=' + p_id + '&versionId=' + v_id
        print(f"Get {url}")
        resp = requests.get(url, auth=self.auth)
        if cycle_name is not None:
            c_name = cycle_name
        else:
            c_name = self.cycle_name
        for key, value in resp.json().items():
            try:
                if c_name.strip() == value['name'].strip():
                    return key
            except Exception as e:
                raise Exception(f'Cycle name not found {e.__str__()}')

    def delete_folder_from_cycle(self, folder_id=None, project_id=None, version_id=None, cycle_id=None):
        if folder_id is not None:
            f_id = folder_id
        else:
            f_id = self.get_folder_id()

        if cycle_id is not None:
            c_id = cycle_id
        else:
            c_id = self.get_cycle_id(self.cycle_name)

        if project_id is not None:
            p_id = project_id
        else:
            p_id = self.get_project_id_by_key()

        if version_id is not None:
            v_id = version_id
        else:
            v_id = self.get_version_for_project_id()

        delete_data = {
            "versionId": v_id,
            "cycleId": c_id,
            "projectId": p_id
        }
        headers = {'content-type': 'application/json'}
        url = self.jira_url + f"zapi/latest/folder/{f_id}"

        print(f"Delete {url} with query params {delete_data}")
        resp = requests.delete(url, params=delete_data, headers=headers, auth=self.auth)
        if resp.status_code != 200:
            raise Exception(f'Could not delete folder id {folder_id} for versionId: {version_id}, cycleId: {cycle_id}, '
                            f'projectId: {project_id}\nResponse: {resp.text}')

    def get_folder_id(self, cycle_id=None, project_id=None, version_id=None, folder_name=None):
        if cycle_id is not None:
            c_id = cycle_id
        else:
            c_id = self.get_cycle_id(self.cycle_name)

        if project_id is not None:
            p_id = project_id
        else:
            p_id = self.get_project_id_by_key()

        if version_id is not None:
            v_id = version_id
        else:
            v_id = self.get_version_for_project_id()

        url = self.jira_url + 'zapi/latest/cycle/' + c_id + '/folders?projectId=' + p_id + '&versionId=' + v_id

        if folder_name:
            f_name = folder_name
        else:
            f_name = self.folder_name

        print(f"Get {url}")
        resp = requests.get(url, auth=self.auth)
        for folder in resp.json():
            if folder['folderName'] == f_name:
                return folder['folderId']

    def create_folder_under_cycle(self, folder_name, cycle_id=None):
        url = self.jira_url + 'zapi/latest/folder/create'

        if cycle_id is not None:
            c_id = cycle_id
        else:
            c_id = self.get_cycle_id(self.cycle_name)
        project_id = self.get_project_id_by_key()
        version_id = self.get_version_for_project_id()
        post_data = json.dumps({
            "versionId": version_id,
            "cycleId": c_id,
            "projectId": project_id,
            "name": folder_name
        })
        headers = {'content-type': 'application/json'}

        print(f"Post {url} with data {post_data}")
        resp = requests.post(url, data=post_data, headers=headers, auth=self.auth)
        if resp.status_code == 200:
            return self.get_folder_id(cycle_id=cycle_id, project_id=project_id, version_id=version_id,
                                      folder_name=folder_name)
        else:
            raise Exception(f'Unable to create folder {resp.content}')



import concurrent.futures
import time
from datetime import date

from .constants.cli_constants import CliConstants
from .model.execution_status import ExecutionStatus
from .model.test_status import TestStatus


class ZephyrUploader:
    def __init__(self, zephyr_service):
        """
        Zephyr uploader class takes a zephyr config and uploads the results in jira zephyr
        :param zephyr_service:
        """
        self.zephyr_service = zephyr_service
        self.config = self.zephyr_service.get_zephyr_config()

    def upload_jira_zephyr(self, excel_data):
        folder_name_with_timestamp = self.config.get(CliConstants.FOLDER_NAME.value) + "_" + date.today().strftime(
            "%Y-%m-%d")

        project_id = self.zephyr_service.get_project_id_by_key(self.config.get(CliConstants.PROJECT_KEY.value))
        version_id = self.zephyr_service.get_version_for_project_id(self.config.get(CliConstants.RELEASE_VERSION.value),
                                                                    project_id=project_id)
        cycle_id = self.zephyr_service.get_cycle_id(self.config.get(CliConstants.TEST_CYCLE.value), project_id,
                                                    version_id)
        folder_id = self.zephyr_service.get_folder_id(folder_name=folder_name_with_timestamp, cycle_id=cycle_id,
                                                      project_id=project_id, version_id=version_id)

        if folder_id is not None and self.config.get(CliConstants.RECREATE_FOLDER.value):
            self.zephyr_service.delete_folder_from_cycle(folder_id=folder_id, project_id=project_id,
                                                         version_id=version_id, cycle_id=cycle_id)
            time.sleep(5)
            folder_id = self.zephyr_service.create_folder_under_cycle(folder_name=folder_name_with_timestamp)

        if folder_id is None:
            folder_id = self.zephyr_service.create_folder_under_cycle(folder_name=folder_name_with_timestamp)

        zephyr_meta_info = {
            "cycleId": cycle_id,
            "projectId": project_id,
            "versionId": version_id,
            "folderId": folder_id,
        }
        self.__upload_jira_zephyr_concurrent(excel_data=excel_data, zephyr_meta_info=zephyr_meta_info)

    def __create_and_update_zephyr_execution(self, row, zephyr_meta_info):
        jira_id = row[0]
        issue_id = self.zephyr_service.get_issue_by_key(jira_id)
        execution_id = self.zephyr_service.create_new_execution(issue_id=issue_id, zephyr_meta_info=zephyr_meta_info)
        if row[self.config.get(CliConstants.EXECUTION_STATUS_COLUMN.value)] == ExecutionStatus.SUCCESS.value:
            self.zephyr_service.update_execution(execution_id, TestStatus.PASSED.value,
                                                 row[self.config.get(CliConstants.COMMENTS_COLUMN.value)])
        elif row[self.config.get(CliConstants.EXECUTION_STATUS_COLUMN.value)] == ExecutionStatus.FAILURE.value:
            self.zephyr_service.update_execution(execution_id, TestStatus.FAILED.value,
                                                 row[self.config.get(CliConstants.COMMENTS_COLUMN.value)])
        else:
            self.zephyr_service.update_execution(execution_id, TestStatus.NOT_EXECUTED.value,
                                                 row[self.config.get(CliConstants.COMMENTS_COLUMN.value)])

    def __upload_jira_zephyr_concurrent(self, excel_data, zephyr_meta_info):
        max_workers = self.config.get(CliConstants.NO_OF_THREADS.value)
        with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
            {executor.submit(self.__create_and_update_zephyr_execution, row, zephyr_meta_info): row for row in
             excel_data}



"""Upload test executions in Jira Zephyr
Import the `ZephyrUploader` class to upload zephyr test executions:
    >>> from zephyr_uploader.zephyr_uploader import ZephyrUploader
    >>> from zephyr_uploader.zephyr_service import ZephyrService
    >>> from zephyr_uploader.model.zephyr_config_dict import ZephyrConfigurer
    >>> config_dict = {
        CliConstants.USERNAME.value: username,
        CliConstants.PASSWORD.value: password,
        CliConstants.JIRA_URL.value: jira_url,
        CliConstants.TEST_CYCLE.value: test_cycle,
        CliConstants.PROJECT_KEY.value: project_key,
        CliConstants.RELEASE_VERSION.value: release_version,
        CliConstants.REPORT_PATH.value: report_path,
        CliConstants.FOLDER_NAME.value: folder_name,
        CliConstants.NO_OF_THREADS.value: no_of_threads,
        CliConstants.RECREATE_FOLDER.value: recreate_folder,
        CliConstants.COMMENTS_COLUMN.value: comments_column,
        CliConstants.EXECUTION_STATUS_COLUMN.value: execution_status_column
    }
    >>> # config_dict = EnvLoader().get_zephyr_config_from_env() # in case a file 'environment.properties is used
    >>> zephyr_configurer = ZephyrConfigurer(config_dict)
    >>> zephyr_configurer.validate()
    >>> zephyr_service = ZephyrService(zephyr_configurer=zephyr_configurer)
    >>> zephyr_uploader = ZephyrUploader(zephyr_service)
    >>> sheet = pyexcel.get_sheet(file_name=zephyr_configurer.get_config().get(CliConstants.REPORT_PATH.value))
    >>>    excel_data = sheet.to_array()
    >>> zephyr_uploader.upload_jira_zephyr(excel_data=excel_data)

See https://github.com/estuaryoss/test-libs-python/tree/master/zephyr_uploader for more information
"""



# -*- coding: utf-8 -*-
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zephyr_uploader",
    version="1.0.0",
    author="Catalin Dinuta",
    author_email="constantin.dinuta@gmail.com",
    description="Upload test execution results to Jira Zephyr. The input is an Excel document.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/estuaryoss/test-libs-python/tree/master/zephyr_uploader",
    packages=["zephyr_uploader"],
    install_requires=[
        "click",
        "pyexcel"
        "requests~=2.25.1"
    ],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True
)



#!/usr/bin/env python3

import click

__author__ = "Catalin Dinuta"

import pyexcel

from .env.env_loader import EnvLoader
from .zephyr_service import ZephyrService
from .zephyr_uploader import ZephyrUploader
from .constants.exit_constants import ExitConstants
from .model.zephyr_config import ZephyrConfigurer
from .constants.cli_constants import CliConstants


@click.command()
@click.option('--username', help='The username used to log in Jira. E.g. auto-robot')
@click.option('--password', help='The password used to log in Jira. E.g. passw0rd123!')
@click.option('--jira_url', help='The jira url REST endpoint used to submit the results, including the last /. '
                                 'E.g. http://jira.yourcompany.com/rest/')
@click.option('--project_key', help='The project key in Jira. E.g. AIP')
@click.option('--release_version', help='The release version. E.g. 1.2-UP2020-4')
@click.option('--test_cycle', help='The test cycle. E.g. Regression_Automated')
@click.option('--report_path', help='The Excel report path on the disk. E.g. Results.xls')
@click.option('--no_of_threads', default=10,
              help='The number of threads to be used to upload the test executions. E.g. 10')
@click.option('--recreate_folder', default=False, help='Recreate the folder under the test cycle or not. '
                                                       'E.g. true. Default: false')
@click.option('--folder_name', help='The release version. E.g. centos7-mysql8-SNAPSHOT. Default: default')
@click.option('--execution_status_column', default=6, help='The execution status column which contains the keywords '
                                                           'SUCCESS/FAILURE. E.g. 10. Default: 6')
@click.option('--comments_column', default=8,
              help='The comments column, for example the link log the logs for the test.'
                   ' E.g. 11. Default: 8')
def cli(username, password, jira_url, project_key, release_version, test_cycle, report_path, no_of_threads,
        recreate_folder, folder_name, execution_status_column, comments_column):
    zephyr_config_dict = EnvLoader().get_zephyr_config_from_env()
    zephyr_configurer = ZephyrConfigurer(zephyr_config_dict)

    zephyr_config_cli = {
        CliConstants.USERNAME.value: username,
        CliConstants.PASSWORD.value: password,
        CliConstants.JIRA_URL.value: jira_url,
        CliConstants.TEST_CYCLE.value: test_cycle,
        CliConstants.PROJECT_KEY.value: project_key,
        CliConstants.RELEASE_VERSION.value: release_version,
        CliConstants.REPORT_PATH.value: report_path,
        CliConstants.FOLDER_NAME.value: folder_name,
        CliConstants.NO_OF_THREADS.value: no_of_threads,
        CliConstants.RECREATE_FOLDER.value: recreate_folder,
        CliConstants.COMMENTS_COLUMN.value: comments_column,
        CliConstants.EXECUTION_STATUS_COLUMN.value: execution_status_column
    }

    zephyr_configurer.override_or_set_default(zephyr_config_cli)
    zephyr_configurer.validate()

    try:
        sheet = pyexcel.get_sheet(file_name=zephyr_configurer.get_config().get(CliConstants.REPORT_PATH.value))
        excel_data = sheet.to_array()
        zephyr_uploader = ZephyrUploader(ZephyrService(zephyr_configurer=zephyr_configurer))
        zephyr_uploader.upload_jira_zephyr(excel_data=excel_data)
    except Exception as e:
        click.echo(e.__str__())
        exit(ExitConstants.FAILURE.value)

    exit(ExitConstants.SUCCESS.value)


if __name__ == "__main__":
    cli()
